'use strict';

var AST = require('@eslint-react/ast');
var core = require('@eslint-react/core');
var eff = require('@eslint-react/eff');
var shared = require('@eslint-react/shared');
var VAR5 = require('@eslint-react/var');
var types = require('@typescript-eslint/types');
var tsPattern = require('ts-pattern');

function _interopNamespace(e) {
  if (e && e.__esModule) return e;
  var n = Object.create(null);
  if (e) {
    Object.keys(e).forEach(function (k) {
      if (k !== 'default') {
        var d = Object.getOwnPropertyDescriptor(e, k);
        Object.defineProperty(n, k, d.get ? d : {
          enumerable: true,
          get: function () { return e[k]; }
        });
      }
    });
  }
  n.default = e;
  return Object.freeze(n);
}

var AST__namespace = /*#__PURE__*/_interopNamespace(AST);
var VAR5__namespace = /*#__PURE__*/_interopNamespace(VAR5);

// package.json
var name = "eslint-plugin-react-hooks-extra";
var version = "1.24.0";
var createRule = shared.createRuleForPlugin("hooks-extra");
function isFromHookCall(name2, context, settings, predicate = eff.returnTrue) {
  const hookAlias = settings.additionalHooks[name2] ?? [];
  return (topLevelId) => {
    const variable = VAR5__namespace.findVariable(topLevelId, context.sourceCode.getScope(topLevelId));
    const variableNode = VAR5__namespace.getVariableNode(variable, 0);
    if (variableNode == null) return false;
    if (variableNode.type !== types.AST_NODE_TYPES.CallExpression) return false;
    if (!core.isReactHookCallWithNameAlias(name2, context, hookAlias)(variableNode)) return false;
    return predicate(topLevelId, variableNode);
  };
}
function isFromUseStateCall(context, settings) {
  const predicate = (topLevelId, call) => {
    const { parent } = call;
    if (!("id" in parent && parent.id?.type === types.AST_NODE_TYPES.ArrayPattern)) {
      return true;
    }
    return parent.id.elements.findIndex((e) => e?.type === types.AST_NODE_TYPES.Identifier && e.name === topLevelId.name) === 1;
  };
  return isFromHookCall("useState", context, settings, predicate);
}
function isSetFunctionCall(context, settings) {
  const isIdFromUseStateCall = isFromUseStateCall(context, settings);
  return (node) => {
    switch (node.callee.type) {
      // const data = useState();
      // data.at(1)();
      case types.AST_NODE_TYPES.CallExpression: {
        const { callee } = node.callee;
        if (callee.type !== types.AST_NODE_TYPES.MemberExpression) {
          return false;
        }
        if (!("name" in callee.object)) {
          return false;
        }
        const isAt = callee.property.type === types.AST_NODE_TYPES.Identifier && callee.property.name === "at";
        const [index] = node.callee.arguments;
        if (!isAt || index == null) {
          return false;
        }
        const indexScope = context.sourceCode.getScope(node);
        const indexValue = VAR5__namespace.toStaticValue({
          kind: "lazy",
          node: index,
          initialScope: indexScope
        }).value;
        return indexValue === 1 && isIdFromUseStateCall(callee.object);
      }
      // const [data, setData] = useState();
      // setData();
      case types.AST_NODE_TYPES.Identifier: {
        return isIdFromUseStateCall(node.callee);
      }
      // const data = useState();
      // data[1]();
      case types.AST_NODE_TYPES.MemberExpression: {
        if (!("name" in node.callee.object)) {
          return false;
        }
        const property = node.callee.property;
        const propertyScope = context.sourceCode.getScope(node);
        const propertyValue = VAR5__namespace.toStaticValue({
          kind: "lazy",
          node: property,
          initialScope: propertyScope
        }).value;
        return propertyValue === 1 && isIdFromUseStateCall(node.callee.object);
      }
      default: {
        return false;
      }
    }
  };
}
function isThenCall(node) {
  return node.callee.type === types.AST_NODE_TYPES.MemberExpression && node.callee.property.type === types.AST_NODE_TYPES.Identifier && node.callee.property.name === "then";
}
function isVariableDeclaratorFromHookCall(node) {
  if (node.type !== types.AST_NODE_TYPES.VariableDeclarator) {
    return false;
  }
  if (node.id.type !== types.AST_NODE_TYPES.Identifier) {
    return false;
  }
  if (node.init?.type !== types.AST_NODE_TYPES.CallExpression) {
    return false;
  }
  switch (node.init.callee.type) {
    case types.AST_NODE_TYPES.Identifier:
      return core.isReactHookName(node.init.callee.name);
    case types.AST_NODE_TYPES.MemberExpression:
      return node.init.callee.property.type === types.AST_NODE_TYPES.Identifier && core.isReactHookName(node.init.callee.property.name);
    default:
      return false;
  }
}

// src/rules/no-direct-set-state-in-use-effect.ts
var RULE_NAME = "no-direct-set-state-in-use-effect";
var RULE_FEATURES = [
  "CHK"
];
var no_direct_set_state_in_use_effect_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow direct calls to the 'set' function of 'useState' in 'useEffect'",
      [Symbol.for("rule_features")]: RULE_FEATURES
    },
    messages: {
      noDirectSetStateInUseEffect: "Do not call the 'set' function '{{name}}' of 'useState' directly in 'useEffect'."
    },
    schema: []
  },
  name: RULE_NAME,
  create(context) {
    if (!/use\w*Effect/u.test(context.sourceCode.text)) {
      return {};
    }
    const settings = shared.getSettingsFromContext(context);
    const additionalHooks = settings.additionalHooks;
    const isUseEffectLikeCall = core.isReactHookCallWithNameAlias("useEffect", context, additionalHooks.useEffect ?? []);
    const isUseStateCall2 = core.isReactHookCallWithNameAlias("useState", context, additionalHooks.useState ?? []);
    const isUseMemoCall2 = core.isReactHookCallWithNameAlias("useMemo", context, additionalHooks.useMemo ?? []);
    const isUseCallbackCall2 = core.isReactHookCallWithNameAlias("useCallback", context, additionalHooks.useCallback ?? []);
    const isSetStateCall = isSetFunctionCall(context, settings);
    const isIdFromUseStateCall = isFromUseStateCall(context, settings);
    const functionEntries = [];
    const setupFunctionRef = { current: null };
    const setupFunctionIdentifiers = [];
    const indFunctionCalls = [];
    const indSetStateCalls = /* @__PURE__ */ new Map();
    const indSetStateCallsInUseEffectArg0 = /* @__PURE__ */ new Map();
    const indSetStateCallsInUseEffectSetup = /* @__PURE__ */ new Map();
    const indSetStateCallsInUseMemoOrCallback = /* @__PURE__ */ new Map();
    const onSetupFunctionEnter = (node) => {
      setupFunctionRef.current = node;
    };
    const onSetupFunctionExit = (node) => {
      if (setupFunctionRef.current === node) {
        setupFunctionRef.current = null;
      }
    };
    function isFunctionOfUseEffectSetup(node) {
      return node.parent?.type === types.AST_NODE_TYPES.CallExpression && node.parent.callee !== node && isUseEffectLikeCall(node.parent);
    }
    function getCallKind(node) {
      return tsPattern.match(node).when(isUseStateCall2, () => "useState").when(isUseEffectLikeCall, () => "useEffect").when(isSetStateCall, () => "setState").when(isThenCall, () => "then").otherwise(() => "other");
    }
    function getFunctionKind(node) {
      return tsPattern.match(node).when(isFunctionOfUseEffectSetup, () => "setup").when(AST__namespace.isFunctionOfImmediatelyInvoked, () => "immediate").otherwise(() => "other");
    }
    return {
      ":function"(node) {
        const kind = getFunctionKind(node);
        functionEntries.push({ kind, node });
        if (kind === "setup") {
          onSetupFunctionEnter(node);
        }
      },
      ":function:exit"(node) {
        const { kind } = functionEntries.at(-1) ?? {};
        if (kind === "setup") {
          onSetupFunctionExit(node);
        }
        functionEntries.pop();
      },
      CallExpression(node) {
        const setupFunction = setupFunctionRef.current;
        const pEntry = functionEntries.at(-1);
        if (pEntry == null || pEntry.node.async) {
          return;
        }
        tsPattern.match(getCallKind(node)).with("setState", () => {
          switch (true) {
            case (pEntry.node === setupFunction || pEntry.kind === "immediate"): {
              context.report({
                messageId: "noDirectSetStateInUseEffect",
                node,
                data: {
                  name: context.sourceCode.getText(node.callee)
                }
              });
              return;
            }
            default: {
              const vd = AST__namespace.findParentNode(node, isVariableDeclaratorFromHookCall);
              if (vd == null) eff.getOrUpdate(indSetStateCalls, pEntry.node, () => []).push(node);
              else eff.getOrUpdate(indSetStateCallsInUseMemoOrCallback, vd.init, () => []).push(node);
            }
          }
        }).with("useEffect", () => {
          if (AST__namespace.isFunction(node.arguments.at(0))) return;
          setupFunctionIdentifiers.push(...AST__namespace.getNestedIdentifiers(node));
        }).with("other", () => {
          if (pEntry.node !== setupFunction) return;
          indFunctionCalls.push(node);
        }).otherwise(() => eff._);
      },
      Identifier(node) {
        if (node.parent.type === types.AST_NODE_TYPES.CallExpression && node.parent.callee === node) {
          return;
        }
        if (!isIdFromUseStateCall(node)) {
          return;
        }
        switch (node.parent.type) {
          case types.AST_NODE_TYPES.ArrowFunctionExpression: {
            const parent = node.parent.parent;
            if (parent.type !== types.AST_NODE_TYPES.CallExpression) {
              break;
            }
            if (!isUseMemoCall2(parent)) {
              break;
            }
            const vd = AST__namespace.findParentNode(parent, isVariableDeclaratorFromHookCall);
            if (vd != null) {
              eff.getOrUpdate(indSetStateCallsInUseEffectArg0, vd.init, () => []).push(node);
            }
            break;
          }
          case types.AST_NODE_TYPES.CallExpression: {
            if (node !== node.parent.arguments.at(0)) {
              break;
            }
            if (isUseCallbackCall2(node.parent)) {
              const vd = AST__namespace.findParentNode(node.parent, isVariableDeclaratorFromHookCall);
              if (vd != null) {
                eff.getOrUpdate(indSetStateCallsInUseEffectArg0, vd.init, () => []).push(node);
              }
              break;
            }
            if (isUseEffectLikeCall(node.parent)) {
              eff.getOrUpdate(indSetStateCallsInUseEffectSetup, node.parent, () => []).push(node);
            }
          }
        }
      },
      "Program:exit"() {
        const getSetStateCalls = (id, initialScope) => {
          const node = VAR5__namespace.getVariableNode(VAR5__namespace.findVariable(id, initialScope), 0);
          switch (node?.type) {
            case types.AST_NODE_TYPES.ArrowFunctionExpression:
            case types.AST_NODE_TYPES.FunctionDeclaration:
            case types.AST_NODE_TYPES.FunctionExpression:
              return indSetStateCalls.get(node) ?? [];
            case types.AST_NODE_TYPES.CallExpression:
              return indSetStateCallsInUseMemoOrCallback.get(node) ?? indSetStateCallsInUseEffectArg0.get(node) ?? [];
          }
          return [];
        };
        for (const [, calls] of indSetStateCallsInUseEffectSetup) {
          for (const call of calls) {
            context.report({
              messageId: "noDirectSetStateInUseEffect",
              node: call,
              data: { name: call.name }
            });
          }
        }
        for (const { callee } of indFunctionCalls) {
          if (!("name" in callee)) {
            continue;
          }
          const { name: name2 } = callee;
          const setStateCalls = getSetStateCalls(name2, context.sourceCode.getScope(callee));
          for (const setStateCall of setStateCalls) {
            context.report({
              messageId: "noDirectSetStateInUseEffect",
              node: setStateCall,
              data: {
                name: AST__namespace.toReadableNodeName(setStateCall, (n) => context.sourceCode.getText(n))
              }
            });
          }
        }
        for (const id of setupFunctionIdentifiers) {
          const setStateCalls = getSetStateCalls(id.name, context.sourceCode.getScope(id));
          for (const setStateCall of setStateCalls) {
            context.report({
              messageId: "noDirectSetStateInUseEffect",
              node: setStateCall,
              data: {
                name: AST__namespace.toReadableNodeName(setStateCall, (n) => context.sourceCode.getText(n))
              }
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME2 = "no-direct-set-state-in-use-layout-effect";
var RULE_FEATURES2 = [
  "CHK"
];
var no_direct_set_state_in_use_layout_effect_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow direct calls to the 'set' function of 'useState' in 'useLayoutEffect'",
      [Symbol.for("rule_features")]: RULE_FEATURES2
    },
    messages: {
      noDirectSetStateInUseLayoutEffect: "Do not call the 'set' function '{{name}}' of 'useState' directly in 'useLayoutEffect'."
    },
    schema: []
  },
  name: RULE_NAME2,
  create(context) {
    if (!/use\w*Effect/u.test(context.sourceCode.text)) {
      return {};
    }
    const settings = shared.getSettingsFromContext(context);
    const additionalHooks = settings.additionalHooks;
    const isUseLayoutEffectLikeCall = core.isReactHookCallWithNameAlias(
      "useLayoutEffect",
      context,
      additionalHooks.useLayoutEffect ?? []
    );
    const isUseStateCall2 = core.isReactHookCallWithNameAlias("useState", context, additionalHooks.useState ?? []);
    const isUseMemoCall2 = core.isReactHookCallWithNameAlias("useMemo", context, additionalHooks.useMemo ?? []);
    const isUseCallbackCall2 = core.isReactHookCallWithNameAlias("useCallback", context, additionalHooks.useCallback ?? []);
    const isSetStateCall = isSetFunctionCall(context, settings);
    const isIdFromUseStateCall = isFromUseStateCall(context, settings);
    const functionEntries = [];
    const setupFunctionRef = { current: null };
    const setupFunctionIdentifiers = [];
    const indFunctionCalls = [];
    const indSetStateCalls = /* @__PURE__ */ new Map();
    const indSetStateCallsInUseLayoutEffectArg0 = /* @__PURE__ */ new Map();
    const indSetStateCallsInUseLayoutEffectSetup = /* @__PURE__ */ new Map();
    const indSetStateCallsInUseMemoOrCallback = /* @__PURE__ */ new Map();
    const onSetupFunctionEnter = (node) => {
      setupFunctionRef.current = node;
    };
    const onSetupFunctionExit = (node) => {
      if (setupFunctionRef.current === node) {
        setupFunctionRef.current = null;
      }
    };
    function isFunctionOfUseEffectSetup(node) {
      return node.parent?.type === types.AST_NODE_TYPES.CallExpression && node.parent.callee !== node && isUseLayoutEffectLikeCall(node.parent);
    }
    function getCallKind(node) {
      return tsPattern.match(node).when(isUseStateCall2, () => "useState").when(isUseLayoutEffectLikeCall, () => "useLayoutEffect").when(isSetStateCall, () => "setState").when(isThenCall, () => "then").otherwise(() => "other");
    }
    function getFunctionKind(node) {
      return tsPattern.match(node).when(isFunctionOfUseEffectSetup, () => "setup").when(AST__namespace.isFunctionOfImmediatelyInvoked, () => "immediate").otherwise(() => "other");
    }
    return {
      ":function"(node) {
        const kind = getFunctionKind(node);
        functionEntries.push({ kind, node });
        if (kind === "setup") {
          onSetupFunctionEnter(node);
        }
      },
      ":function:exit"(node) {
        const { kind } = functionEntries.at(-1) ?? {};
        if (kind === "setup") {
          onSetupFunctionExit(node);
        }
        functionEntries.pop();
      },
      CallExpression(node) {
        const setupFunction = setupFunctionRef.current;
        const pEntry = functionEntries.at(-1);
        if (pEntry == null || pEntry.node.async) {
          return;
        }
        tsPattern.match(getCallKind(node)).with("setState", () => {
          switch (true) {
            case (pEntry.node === setupFunction || pEntry.kind === "immediate"): {
              context.report({
                messageId: "noDirectSetStateInUseLayoutEffect",
                node,
                data: {
                  name: context.sourceCode.getText(node.callee)
                }
              });
              return;
            }
            default: {
              const vd = AST__namespace.findParentNode(node, isVariableDeclaratorFromHookCall);
              if (vd == null) eff.getOrUpdate(indSetStateCalls, pEntry.node, () => []).push(node);
              else eff.getOrUpdate(indSetStateCallsInUseMemoOrCallback, vd.init, () => []).push(node);
            }
          }
        }).with("useLayoutEffect", () => {
          if (AST__namespace.isFunction(node.arguments.at(0))) return;
          setupFunctionIdentifiers.push(...AST__namespace.getNestedIdentifiers(node));
        }).with("other", () => {
          if (pEntry.node !== setupFunction) return;
          indFunctionCalls.push(node);
        }).otherwise(() => eff._);
      },
      Identifier(node) {
        if (node.parent.type === types.AST_NODE_TYPES.CallExpression && node.parent.callee === node) {
          return;
        }
        if (!isIdFromUseStateCall(node)) {
          return;
        }
        switch (node.parent.type) {
          case types.AST_NODE_TYPES.ArrowFunctionExpression: {
            const parent = node.parent.parent;
            if (parent.type !== types.AST_NODE_TYPES.CallExpression) {
              break;
            }
            if (!isUseMemoCall2(parent)) {
              break;
            }
            const vd = AST__namespace.findParentNode(parent, isVariableDeclaratorFromHookCall);
            if (vd != null) {
              eff.getOrUpdate(indSetStateCallsInUseLayoutEffectArg0, vd.init, () => []).push(node);
            }
            break;
          }
          case types.AST_NODE_TYPES.CallExpression: {
            if (node !== node.parent.arguments.at(0)) {
              break;
            }
            if (isUseCallbackCall2(node.parent)) {
              const vd = AST__namespace.findParentNode(node.parent, isVariableDeclaratorFromHookCall);
              if (vd != null) {
                eff.getOrUpdate(indSetStateCallsInUseLayoutEffectArg0, vd.init, () => []).push(node);
              }
              break;
            }
            if (isUseLayoutEffectLikeCall(node.parent)) {
              eff.getOrUpdate(indSetStateCallsInUseLayoutEffectSetup, node.parent, () => []).push(node);
            }
          }
        }
      },
      "Program:exit"() {
        const getSetStateCalls = (id, initialScope) => {
          const node = VAR5__namespace.getVariableNode(VAR5__namespace.findVariable(id, initialScope), 0);
          switch (node?.type) {
            case types.AST_NODE_TYPES.ArrowFunctionExpression:
            case types.AST_NODE_TYPES.FunctionDeclaration:
            case types.AST_NODE_TYPES.FunctionExpression:
              return indSetStateCalls.get(node) ?? [];
            case types.AST_NODE_TYPES.CallExpression:
              return indSetStateCallsInUseMemoOrCallback.get(node) ?? indSetStateCallsInUseLayoutEffectArg0.get(node) ?? [];
          }
          return [];
        };
        for (const [, calls] of indSetStateCallsInUseLayoutEffectSetup) {
          for (const call of calls) {
            context.report({
              messageId: "noDirectSetStateInUseLayoutEffect",
              node: call,
              data: { name: call.name }
            });
          }
        }
        for (const { callee } of indFunctionCalls) {
          if (!("name" in callee)) {
            continue;
          }
          const { name: name2 } = callee;
          const setStateCalls = getSetStateCalls(name2, context.sourceCode.getScope(callee));
          for (const setStateCall of setStateCalls) {
            context.report({
              messageId: "noDirectSetStateInUseLayoutEffect",
              node: setStateCall,
              data: {
                name: AST__namespace.toReadableNodeName(setStateCall, (n) => context.sourceCode.getText(n))
              }
            });
          }
        }
        for (const id of setupFunctionIdentifiers) {
          const setStateCalls = getSetStateCalls(id.name, context.sourceCode.getScope(id));
          for (const setStateCall of setStateCalls) {
            context.report({
              messageId: "noDirectSetStateInUseLayoutEffect",
              node: setStateCall,
              data: {
                name: AST__namespace.toReadableNodeName(setStateCall, (n) => context.sourceCode.getText(n))
              }
            });
          }
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME3 = "no-unnecessary-use-callback";
var RULE_FEATURES3 = [
  "CHK"
];
var no_unnecessary_use_callback_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow unnecessary usage of 'useCallback'",
      [Symbol.for("rule_features")]: RULE_FEATURES3
    },
    messages: {
      noUnnecessaryUseCallback: "An 'useCallback' with empty deps and no references to the component scope may be unnecessary."
    },
    schema: []
  },
  name: RULE_NAME3,
  create(context) {
    if (!context.sourceCode.text.includes("use")) {
      return {};
    }
    const alias = shared.getSettingsFromContext(context).additionalHooks.useCallback ?? [];
    return {
      CallExpression(node) {
        if (!core.isReactHookCall(node)) {
          return;
        }
        const initialScope = context.sourceCode.getScope(node);
        if (!core.isUseCallbackCall(node, context) && !alias.some(core.isReactHookCallWithNameLoose(node))) {
          return;
        }
        const scope = context.sourceCode.getScope(node);
        const component = scope.block;
        if (!AST__namespace.isFunction(component)) {
          return;
        }
        const [arg0, arg1] = node.arguments;
        if (arg0 == null || arg1 == null) {
          return;
        }
        const hasEmptyDeps = tsPattern.match(arg1).with({ type: types.AST_NODE_TYPES.ArrayExpression }, (n) => n.elements.length === 0).with({ type: types.AST_NODE_TYPES.Identifier }, (n) => {
          const variable = VAR5__namespace.findVariable(n.name, initialScope);
          const variableNode = VAR5__namespace.getVariableNode(variable, 0);
          if (variableNode?.type !== types.AST_NODE_TYPES.ArrayExpression) {
            return false;
          }
          return variableNode.elements.length === 0;
        }).otherwise(() => false);
        if (!hasEmptyDeps) {
          return;
        }
        const arg0Node = tsPattern.match(arg0).with({ type: types.AST_NODE_TYPES.ArrowFunctionExpression }, (n) => {
          if (n.body.type === types.AST_NODE_TYPES.ArrowFunctionExpression) {
            return n.body;
          }
          return n;
        }).with({ type: types.AST_NODE_TYPES.FunctionExpression }, eff.identity).with({ type: types.AST_NODE_TYPES.Identifier }, (n) => {
          const variable = VAR5__namespace.findVariable(n.name, initialScope);
          const variableNode = VAR5__namespace.getVariableNode(variable, 0);
          if (variableNode?.type !== types.AST_NODE_TYPES.ArrowFunctionExpression && variableNode?.type !== types.AST_NODE_TYPES.FunctionExpression) {
            return eff._;
          }
          return variableNode;
        }).otherwise(() => eff._);
        if (arg0Node == null) return;
        const arg0NodeScope = context.sourceCode.getScope(arg0Node);
        const arg0NodeReferences = VAR5__namespace.getChidScopes(arg0NodeScope).flatMap((x) => x.references);
        const isReferencedToComponentScope = arg0NodeReferences.some((x) => x.resolved?.scope.block === component);
        if (!isReferencedToComponentScope) {
          context.report({
            messageId: "noUnnecessaryUseCallback",
            node
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME4 = "no-unnecessary-use-memo";
var RULE_FEATURES4 = [
  "CHK"
];
var no_unnecessary_use_memo_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow unnecessary usage of 'useMemo'",
      [Symbol.for("rule_features")]: RULE_FEATURES4
    },
    messages: {
      noUnnecessaryUseMemo: "An 'useMemo' with empty deps and no references to the component scope may be unnecessary."
    },
    schema: []
  },
  name: RULE_NAME4,
  create(context) {
    if (!context.sourceCode.text.includes("use")) {
      return {};
    }
    const alias = shared.getSettingsFromContext(context).additionalHooks.useMemo ?? [];
    return {
      CallExpression(node) {
        if (!core.isReactHookCall(node)) {
          return;
        }
        const initialScope = context.sourceCode.getScope(node);
        if (!core.isUseMemoCall(node, context) && !alias.some(core.isReactHookCallWithNameLoose(node))) {
          return;
        }
        const scope = context.sourceCode.getScope(node);
        const component = scope.block;
        if (!AST__namespace.isFunction(component)) {
          return;
        }
        const [arg0, arg1] = node.arguments;
        if (arg0 == null || arg1 == null) {
          return;
        }
        const hasCallInArg0 = AST__namespace.isFunction(arg0) && [...AST__namespace.getNestedCallExpressions(arg0.body), ...AST__namespace.getNestedNewExpressions(arg0.body)].length > 0;
        if (hasCallInArg0) {
          return;
        }
        const hasEmptyDeps = tsPattern.match(arg1).with({ type: types.AST_NODE_TYPES.ArrayExpression }, (n) => n.elements.length === 0).with({ type: types.AST_NODE_TYPES.Identifier }, (n) => {
          const variable = VAR5__namespace.findVariable(n.name, initialScope);
          const variableNode = VAR5__namespace.getVariableNode(variable, 0);
          if (variableNode?.type !== types.AST_NODE_TYPES.ArrayExpression) {
            return false;
          }
          return variableNode.elements.length === 0;
        }).otherwise(() => false);
        if (!hasEmptyDeps) {
          return;
        }
        const arg0Node = tsPattern.match(arg0).with({ type: types.AST_NODE_TYPES.ArrowFunctionExpression }, (n) => {
          if (n.body.type === types.AST_NODE_TYPES.ArrowFunctionExpression) {
            return n.body;
          }
          return n;
        }).with({ type: types.AST_NODE_TYPES.FunctionExpression }, eff.identity).with({ type: types.AST_NODE_TYPES.Identifier }, (n) => {
          const variable = VAR5__namespace.findVariable(n.name, initialScope);
          const variableNode = VAR5__namespace.getVariableNode(variable, 0);
          if (variableNode?.type !== types.AST_NODE_TYPES.ArrowFunctionExpression && variableNode?.type !== types.AST_NODE_TYPES.FunctionExpression) {
            return eff._;
          }
          return variableNode;
        }).otherwise(() => eff._);
        if (arg0Node == null) return;
        const arg0NodeScope = context.sourceCode.getScope(arg0Node);
        const arg0NodeReferences = VAR5__namespace.getChidScopes(arg0NodeScope).flatMap((x) => x.references);
        const isReferencedToComponentScope = arg0NodeReferences.some((x) => x.resolved?.scope.block === component);
        if (!isReferencedToComponentScope) {
          context.report({
            messageId: "noUnnecessaryUseMemo",
            node
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME5 = "no-useless-custom-hooks";
var RULE_FEATURES5 = [
  "CHK"
];
function isNodeContainsUseCallComments(node, context) {
  return context.sourceCode.getCommentsInside(node).some((comment) => /use\w+\(/u.test(comment.value));
}
var no_useless_custom_hooks_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "enforce custom Hooks to use at least one other hook inside",
      [Symbol.for("rule_features")]: RULE_FEATURES5
    },
    messages: {
      noUselessCustomHooks: "A custom Hook '{{name}}' should use at least one other hook."
    },
    schema: []
  },
  name: RULE_NAME5,
  create(context) {
    const { ctx, listeners } = core.useHookCollector();
    return {
      ...listeners,
      "Program:exit"(node) {
        const allHooks = ctx.getAllHooks(node);
        for (const { name: name2, node: node2, hookCalls } of allHooks.values()) {
          if (AST__namespace.isEmptyFunction(node2)) {
            continue;
          }
          if (hookCalls.length > 0) {
            continue;
          }
          if (isNodeContainsUseCallComments(node2, context)) {
            continue;
          }
          context.report({
            messageId: "noUselessCustomHooks",
            node: node2,
            data: {
              name: name2
            }
          });
        }
      }
    };
  },
  defaultOptions: []
});
var RULE_NAME6 = "prefer-use-state-lazy-initialization";
var RULE_FEATURES6 = [
  "CHK"
];
var ALLOW_LIST = ["Boolean", "String", "Number"];
var prefer_use_state_lazy_initialization_default = createRule({
  meta: {
    type: "problem",
    docs: {
      description: "disallow function calls in 'useState' that aren't wrapped in an initializer function",
      [Symbol.for("rule_features")]: RULE_FEATURES6
    },
    messages: {
      preferUseStateLazyInitialization: "To prevent re-computation, consider using lazy initial state for useState calls that involve function calls. Ex: 'useState(() => getValue())'."
    },
    schema: []
  },
  name: RULE_NAME6,
  create(context) {
    if (!context.sourceCode.text.includes("use")) {
      return {};
    }
    const alias = shared.getSettingsFromContext(context).additionalHooks.useState ?? [];
    return {
      CallExpression(node) {
        if (!core.isReactHookCall(node)) {
          return;
        }
        if (!core.isUseStateCall(node, context) && !alias.some(core.isReactHookCallWithNameLoose(node))) {
          return;
        }
        const [useStateInput] = node.arguments;
        if (useStateInput == null) {
          return;
        }
        const nestedCallExpressions = AST__namespace.getNestedCallExpressions(useStateInput);
        const hasFunctionCall = nestedCallExpressions.some((n) => {
          return "name" in n.callee && !ALLOW_LIST.includes(n.callee.name);
        });
        const hasNewCall = AST__namespace.getNestedNewExpressions(useStateInput).some((n) => {
          return "name" in n.callee && !ALLOW_LIST.includes(n.callee.name);
        });
        if (!hasFunctionCall && !hasNewCall) {
          return;
        }
        context.report({
          messageId: "preferUseStateLazyInitialization",
          node: useStateInput
        });
      }
    };
  },
  defaultOptions: []
});

// src/index.ts
var index_default = {
  meta: {
    name,
    version
  },
  rules: {
    "no-direct-set-state-in-use-effect": no_direct_set_state_in_use_effect_default,
    "no-direct-set-state-in-use-layout-effect": no_direct_set_state_in_use_layout_effect_default,
    "no-unnecessary-use-callback": no_unnecessary_use_callback_default,
    "no-unnecessary-use-memo": no_unnecessary_use_memo_default,
    "no-useless-custom-hooks": no_useless_custom_hooks_default,
    "prefer-use-state-lazy-initialization": prefer_use_state_lazy_initialization_default,
    // Part: deprecated rules
    /** @deprecated Use `no-useless-custom-hooks` instead */
    "ensure-custom-hooks-using-other-hooks": no_useless_custom_hooks_default,
    /** @deprecated Use `no-unnecessary-use-callback` instead */
    "ensure-use-callback-has-non-empty-deps": no_unnecessary_use_callback_default,
    /** @deprecated Use `no-unnecessary-use-memo` instead */
    "ensure-use-memo-has-non-empty-deps": no_unnecessary_use_memo_default,
    /** @deprecated Use `no-useless-custom-hooks` instead */
    "no-redundant-custom-hook": no_useless_custom_hooks_default
  }
};

module.exports = index_default;
